
############
#Objective
############
#This function is to subsample the data and perform LASSO (single time) on the selected samples

##########
#INPUTS:
##########
#data: the dataframe should be arranged in the way such that columns are
#X1,X2,X3....,Xp, status.  Where Xi's are variables and status is the 
#binary outcome (0 or 1)

#fold: fold used in lasso cross validation to select the tuning parameter

#Type: should use 'class' for classification always

#Alpha: 1 for Lasso,0 for ridgeression

#prop: percentage of samples left out for each subsamping

#weights: =TRUE: if weighted version is desired,   =FALSE, otherwise

##########
#OUTPUTS
##########
#lambda: the tuning parameter that within 1 sd of the tuning parameter gives the lowest CV error

#coef: a table shows the name of the selected variables by LASSO and its coefficients

#table: there are a equal proportion of samples from each status left out  and we use the model built on the selected
#subsamples to predict those left out ones. Table contains two columns: column1 is the predicted value and columb2 is
#the true class

#error: misclassification error based on the above table

#Beta: should be a vector of length p+1 and this is the beta coefficients from the LASSO model. 
#Be aware of that the intercept is placed at the end of this vector

##############################################################################






Subsample.w<-function(data,fold,Alpha,prop,weights,family,Type){
  N=dim(data)[1]
  ID=1:N

  
  len<-dim(data)[2]
  
  if(family$family=="binomial"){
###      stratified sampling by classes
      test.ID=NULL
      n.samp1=ceiling(length(ID[data[,len]==1])*prop)
      n.samp0=ceiling(length(ID[data[,len]==0])*prop)
      test.ID=sample(ID[data[,len]==1],n.samp1)
      
      test.ID=c(test.ID,sample(ID[data[,len]==0],n.samp0))
  }else{
      test.ID=sample(ID,N*prop)
      if(!is.null(weights)){
          if(weights==TRUE){
              warning("weighted regression only supported for binomial variables")
          }
      }
  }

  train.ID=ID[-test.ID]
      
  sel=1:(len-1)

#    which(substring(colnames(data),1,1)=="X"&(!is.na(as.numeric(substring(colnames(data),2)))))
  test.X=data[test.ID,sel]
  train.X=data[-test.ID,sel]
  test.Y=data[test.ID,len]
  train.Y=data[-test.ID,len]
  
  N=length(test.ID) 
  N1=length(which(test.Y==1))
  N0=length(which(test.Y==0))
  
  
  P=dim(train.X)[2]
  
  Beta=numeric(P)
  NonZero_ind=which(apply(train.X,2,sum)!=0)
  Beta[-NonZero_ind]=0
### remove predictors which are all zero from data

  if(family$family=="binomial"){
      train.Y<-as.factor(train.Y)
      test.Y<-as.factor(test.Y)
  }

  
  if(is.null(weights)|weights==FALSE){
      model.cv=glmnet::cv.glmnet(as.matrix(train.X[,NonZero_ind]),train.Y,family=family$family,type.measure=Type,nfolds=fold,alpha=Alpha,standardize=T,parallel=FALSE)
  } else { 
    N1.tr=length(which(train.Y==1))
    N0.tr=length(which(train.Y==0))
    
    if(N1.tr>=N0.tr)w=ifelse(train.Y==1,1,(N1.tr/N0.tr)) else w=ifelse(train.Y==1,(N0.tr/N1.tr),1) 
    model.cv=glmnet::cv.glmnet(as.matrix(train.X[,NonZero_ind]),train.Y,family=family$family,type.measure=Type,nfolds=fold,alpha=Alpha,standardize=T,weights=w,parallel=FALSE)
  }

  min.s=model.cv$lambda.min
  model.pr=stats::predict(model.cv,as.matrix(test.X[,NonZero_ind]),s="lambda.min")
  
  
  Beta[NonZero_ind]=as.vector(stats::coef(model.cv,s=min.s)[-1])
  
  Beta=c(Beta,stats::coef(model.cv,s=min.s)[1])
  
  if(family$family=="binomial"){
      table=cbind( as.numeric(noquote(model.pr)),c(rep(1,N1),rep(0,N0)))
      colnames(table)=c("Predicted Class","True Class")
      if(N1<=N0){
          error.w=((N0/N1)*table[1,2]+table[2,1])/((N0/N1)*(table[1,2]+table[2,2])+(table[1,1]+table[2,1]))
      }else{
          error.w=((N1/N0)*table[2,1]+table[1,2])/((N1/N0)*(table[1,1]+table[2,1])+(table[1,2]+table[2,2]))
      }
  }else{
###Numeric predictor, use MSE
      table=cbind( as.numeric(noquote(model.pr)),test.Y)
      error.w=mean((table[,1]-table[,2])^2)
  }
  
  COEF=cbind(rownames(stats::coef(model.cv))[-which(as.logical(stats::coef(model.cv)==0))],stats::coef(model.cv)[-which(as.logical(stats::coef(model.cv)==0))])
  
  # return(list(lambda=model.cv$lambda.1se,coef=COEF,table=table,Class.Error1=1-sum(table[,1]==table[,2])/(n.samp*6),Class.Error2=1-sum(table[3:8,1]==table[3:8,2])/(n.samp*3),Class.Error.enet=1-sum(table.enet[,1]==table.enet[,2])/(n.samp*4),Class.Error2.enet=1-sum(table.enet[3:8,1]==table.enet[3:8,2])/(n.samp*3)))

  return(list(lambda=min.s,coef=COEF,table=table,Error=error.w,Beta=Beta))
  
}
